@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,9 +19,9 @@
 // ========================================================================
 //
 // File          : $RCSfile: biff.cc,v $
-// Revision      : $Revision: 1.44.2.2 $
-// Revision date : $Date: 2005/12/04 20:05:04 $
-// Author(s)     : Nicolas Rougier
+// Revision      : $Revision: 1.64 $
+// Revision date : $Date: 2006/01/07 22:35:22 $
+// Author(s)     : Nicolas Rougier, Robert Sowada
 // Short         : 
 //
 // This file is part of gnubiff.
@@ -42,22 +42,19 @@
 #  include "ui-applet-gnome.h"
 #endif
 
+#include "apop.h"
 #include "biff.h"
-#include "ui-authentication.h"
-#include "ui-preferences.h"
-#include "ui-properties.h"
-#include "ui-applet-gtk.h"
-#include "ui-popup.h"
-#include "mailbox.h"
 #include "file.h"
+#include "imap4.h"
+#include "mailbox.h"
 #include "maildir.h"
 #include "mh.h"
 #include "mh_basic.h"
 #include "mh_sylpheed.h"
-#include "imap4.h"
 #include "pop3.h"
-#include "apop.h"
-
+#include "signals.h"
+#include "ui-applet-gtk.h"
+#include "ui-applet-systray.h"
 
 // ============================================================================
 //  "C" binding
@@ -99,28 +96,18 @@ extern "C" {
 	}
 }
 
-// ================================================================================
+// ============================================================================
 //  base
-// ================================================================================
+// ============================================================================
 Biff::Biff (guint ui_mode, std::string filename)
 {
-	// Get password table from configure option
-#ifdef USE_PASSWORD
-	passtable_ = PASSWORD_STRING;
-	// Add something (in case the provided string is not long enough)
-	passtable_ += "FEDCBA9876543210";
-	//  and then we remove duplicated characters
-	std::string buffer;
-	for (std::string::size_type i = 0; i < passtable_.size(); i++)
-		if (buffer.find(passtable_[i]) == std::string::npos)
-			buffer += passtable_[i];
-	passtable_ = buffer;
-#endif
-
 	mutex_ = g_mutex_new ();
 
+	// Authentication mutex
+	auth_mutex_ = g_mutex_new ();
+
 	// Add options
-	add_options (OPTGRP_ALL & (~OPTGRP_MAILBOX));
+	add_options (OPTGRP_ALL & (~OPTGRP_MAILBOX), true);
 
 	// Set session specific options
 	if (filename.size() > 0)
@@ -139,29 +126,36 @@ Biff::Biff (guint ui_mode, std::string filename)
 				   value_gchar ("config_file"));
 		mailbox_.push_back (new Mailbox (this));
 	}
+	value ("config_file_loaded", true);
+
+	// Remove all deprecated options
+	remove_options (OPTFLG_DEPRECATED);
+	for (guint i = 0; i < mailbox_.size(); i++)
+		mailbox_[i]->remove_options (OPTFLG_DEPRECATED);
 
 	// Applet
+	switch (ui_mode) {
 #ifdef USE_GNOME
-	if (ui_mode == GNOME_MODE)
+	case MODE_GNOME:
 		applet_ = new AppletGnome (this);
-	else
-		applet_ = new AppletGtk (this);
-#else
-	applet_ = new AppletGtk (this);
+		break;
 #endif
-	applet_->create();
+	case MODE_GTK:
+		applet_ = new AppletGtk (this);
+		break;
+	case MODE_NOGUI:
+		applet_ = new Applet (this);
+		break;
+	case MODE_SYSTEMTRAY:
+		applet_ = new AppletSystray (this);
+		break;
+	default:
+		applet_ = new AppletGtk (this);
+		break;
+	}
 
-	// Preferences
-	preferences_ = new Preferences (this);
-	preferences_->create ();
-
-	// Popup
-	popup_ = new Popup (this);
-	popup_->create();
-
-	// Authentication dialog
-	ui_auth_mutex_ = g_mutex_new ();
-	ui_auth_ = new Authentication ();
+	// Initialize Signals
+	Signals::init_signals (this);
 }
 
 /// Destructor
@@ -169,20 +163,20 @@ Biff::~Biff (void)
 {
 }
 
-// ================================================================================
+// ============================================================================
 //  access
-// ================================================================================
+// ============================================================================
 
 /**
- *  Search in all mailboxes for the mail with id {\em mailid}.
+ *  Search in all mailboxes for the message with id {\em mailid}.
  *
- *  @param  mailid  Gnubiff mail identifier of the mail to find
+ *  @param  mailid  Gnubiff message identifier of the mail to find
  *  @param  mail    Here the header of the found mail is returned. If no mail
  *                  with id {\em mailid} exists, {\em mail} remains unchanged.
  *  @returns        Boolean indicating if a mail exists or not.
  */
 gboolean 
-Biff::find_mail (std::string mailid, Header &mail)
+Biff::find_message (std::string mailid, Header &mail)
 {
 	gboolean ok = false;
 
@@ -195,15 +189,6 @@ Biff::find_mail (std::string mailid, Header &mail)
 	return ok;
 }
 
-guint 
-Biff::size (void)
-{
-	g_mutex_lock (mutex_);
-	guint size = mailbox_.size();
-	g_mutex_unlock (mutex_);
-	return size;
-}
-
 Mailbox *
 Biff::mailbox (guint index)
 {
@@ -226,49 +211,116 @@ Biff::get (guint uin)
 	return find;
 }
 
-void
-Biff::add (Mailbox *mailbox)
+// ============================================================================
+//  main -- mailbox handling
+// ============================================================================
+
+/**
+ *  Add the new mailbox {\em mailbox} for monitoring.
+ *
+ *  @param mailbox Mailbox to be added for monitoring.
+ */
+void 
+Biff::add_mailbox (Mailbox *mailbox)
 {
 	g_mutex_lock (mutex_);
 	mailbox_.push_back (mailbox);
 	g_mutex_unlock (mutex_);
 }
 
-Mailbox *
-Biff::replace (Mailbox *from, Mailbox *to)
+/**
+ *  Get the number of mailboxes that are being monitored.
+ *
+ *  @return   Number of mailboxes.
+ */
+guint 
+Biff::get_number_of_mailboxes (void)
 {
-	Mailbox *inserted = 0;
 	g_mutex_lock (mutex_);
-	for (std::vector<Mailbox *>::iterator i = mailbox_.begin();
-		 i != mailbox_.end(); i++)
-		if ((*i) == from) {
-			(*i) = to;
-
-			if ((preferences_) && (preferences_->selected() == from))
-				preferences_->selected (to);
-			delete from;
-			inserted = to;
-			break;
-		}
+	guint size = mailbox_.size();
 	g_mutex_unlock (mutex_);
-
-	if ((inserted) && (!preferences_
-					   || !GTK_WIDGET_VISIBLE(preferences_->get())))
-		inserted->threaded_start (3);	
-	return inserted;
+	return size;
 }
 
-void
-Biff::remove (Mailbox *mailbox)
+/**
+ *  Get message headers from the messages in the mailboxes. If
+ *  {\em use_max_num} is true, the number of headers is limited to
+ *  {\em max_num}. If there are more message headers available than are to be
+ *  returned, from each mailbox the same number of headers is taken as far as
+ *  possible.
+ *
+ *  @param  use_max_num  If true only a limited amount of headers is returned
+ *                       (the default is false).
+ *  @param  max_num      If {\em use_max_num} is true, this is the maximum
+ *                       number of message headers to be returned (the default
+ *                       is 0).
+ *  @return              Vector of message headers
+ */
+std::vector<Header *> 
+Biff::get_message_headers (gboolean use_max_num, guint max_num)
 {
 	g_mutex_lock (mutex_);
-	for(std::vector<Mailbox *>::iterator i = mailbox_.begin();
-		i != mailbox_.end(); i++)
-		if ((*i) == mailbox) {
-			mailbox_.erase(i);
-			break;
+
+	// Get number of messages in each mailbox
+	guint num_mails = 0, num_boxes = 0;
+	num_boxes = mailbox_.size();
+	std::vector<guint> max (num_boxes);
+	for (guint i = 0; i < num_boxes; i++)
+		num_mails+= max[i] = mailbox_[i]->unreads();
+
+	// Distribute number of message headers
+	std::vector<guint> count (num_boxes, 0);
+	if (use_max_num) {
+		guint index = 0, all = 0;
+		num_mails = std::min (num_mails, max_num);
+		while (all < num_mails) {
+			if (count[index] < max[index]) {
+				count[index]++;
+				all++;
+			}
+			index = (index + 1) % num_boxes;
 		}
+	}
+	else
+		count = max;
+
+	// Put all the headers to be displayed in one vector.
+	// Note: In the mean time some headers may have been deleted, so less than
+	// count[j] headers could be added to the vector
+	std::vector<Header *> headers;
+	for (guint j = 0; j < num_boxes; j++)
+		mailbox_[j]->get_message_headers (headers, true, count[j]);
+
 	g_mutex_unlock (mutex_);
+
+	return headers;
+}
+
+/**
+ *  Get the number of unread messages in all mailboxes.
+ *
+ *  @param  num  Here the number of unread messages is returned.
+ *  @return      This boolean indicates whether there are new messages (true)
+ *               or not (false).
+ */
+gboolean 
+Biff::get_number_of_unread_messages (guint &num)
+{
+	std::vector<class Mailbox *>::iterator mailbox;
+	gboolean newmail = false;
+	num = 0;
+
+	g_mutex_lock (mutex_);
+	mailbox = mailbox_.begin ();
+	while (mailbox != mailbox_.end ()) {
+		if ((*mailbox)->status () == MAILBOX_NEW)
+			newmail = true;
+		num += (*mailbox)->unreads ();
+		mailbox++;
+	}
+	g_mutex_unlock (mutex_);
+
+	return newmail;
 }
 
 /**
@@ -286,7 +338,7 @@ Biff::remove (Mailbox *mailbox)
  * @return          Boolean indicating whether a password could be obtained
  */
 gboolean 
-Biff::password (Mailbox *m)
+Biff::get_password_for_mailbox (Mailbox *m)
 {
 	// Do we know the password already?
 	if (!m->password().empty())
@@ -294,7 +346,7 @@ Biff::password (Mailbox *m)
 
 	// Remark: It's important to block thread before looking at other mailboxes
 	//         since one is maybe asking (using gui) for this password.
-	g_mutex_lock (ui_auth_mutex_);
+	g_mutex_lock (auth_mutex_);
 
 	// Searching other mailboxes
 #if DEBUG
@@ -302,7 +354,7 @@ Biff::password (Mailbox *m)
 			   m->username().c_str(), m->address().c_str(), m->port());
 #endif
 
-	for (guint i = 0; i < size(); i++)
+	for (guint i = 0; i < get_number_of_mailboxes (); i++)
 		if ((mailbox(i) != m) 
 			&& (mailbox(i)->address() == m->address())
 			&& (mailbox(i)->username() == m->username())
@@ -315,14 +367,144 @@ Biff::password (Mailbox *m)
 	// Ask the user if password is still not known
 	if (m->password().empty()) {
 		gdk_threads_enter ();
-		ui_auth_->select (m);
+		applet_->get_password_for_mailbox (m);
 		gdk_threads_leave ();
 	}
 
-	g_mutex_unlock (ui_auth_mutex_);
+	g_mutex_unlock (auth_mutex_);
 	return !m->password().empty();
 }
 
+/**
+ *  Mark all (obtained) messages from all mailboxes as read.
+ */
+void 
+Biff::mark_messages_as_read (void)
+{
+	std::vector<class Mailbox *>::iterator mailbox;
+	g_mutex_lock (mutex_);
+	mailbox = mailbox_.begin ();
+	while (mailbox != mailbox_.end ())
+		(*(mailbox++))->mark_messages_as_read ();
+	g_mutex_unlock (mutex_);
+}
+
+/**
+ *  This function has to been called, when all (obtained) messages have been
+ *  displayed to the user. The status of the mailboxes will be updated.
+ */
+void 
+Biff::messages_displayed (void)
+{
+	std::vector<class Mailbox *>::iterator mailbox;
+
+	g_mutex_lock (mutex_);
+	mailbox = mailbox_.begin ();
+	while (mailbox != mailbox_.end ())
+	  (*(mailbox++))->mail_displayed ();
+	g_mutex_unlock (mutex_);
+}
+
+/**
+ *  Replace the mailbox {\em from} in the list of all mailboxes by the mailbox
+ *  {\em to}. The mailbox {\em from} will be destroyed.
+ *
+ *  @param  from  Mailbox to be replaced.
+ *  @param  to    Mailbox that replaces the mailbox {\em from}.
+ *  @return       NULL, if the mailbox {\em from} doesn't exist, otherwise a
+ *                pointer to the mailbox {\em to}.
+ */
+Mailbox *
+Biff::replace_mailbox (Mailbox *from, Mailbox *to)
+{
+	Mailbox *inserted = NULL;
+
+	g_mutex_lock (mutex_);
+	for (std::vector<Mailbox *>::iterator i = mailbox_.begin();
+		 i != mailbox_.end(); i++)
+		if ((*i) == from) {
+			(*i) = to;
+
+			// Let the applet do necessary things before the mailbox can be
+			// replaced
+			applet_->mailbox_to_be_replaced (from, to);
+
+			delete from;
+			inserted = to;
+			break;
+		}
+	g_mutex_unlock (mutex_);
+
+	// Start monitoring the new mailbox
+	if ((inserted) && (applet_->can_monitor_mailboxes ()))
+		inserted->threaded_start (3);
+	return inserted;
+}
+
+/**
+ *  Remove the mailbox {\em mailbox}.
+ *
+ *  @param mailbox Mailbox to be removed.
+ */
+void 
+Biff::remove_mailbox (Mailbox *mailbox)
+{
+	g_mutex_lock (mutex_);
+	for(std::vector<Mailbox *>::iterator i = mailbox_.begin();
+		i != mailbox_.end(); i++)
+		if ((*i) == mailbox) {
+			mailbox_.erase (i);
+			break;
+		}
+	g_mutex_unlock (mutex_);
+}
+
+/**
+ *  Start monitoring all mailboxes. Optionally the delay {\em delay} can be
+ *  given, so monitoring starts later.
+ *
+ *  @param  delay  Delay in seconds (the default is 0).
+ */
+void 
+Biff::start_monitoring (guint delay)
+{
+#ifdef DEBUG
+	if (delay)
+		g_message ("Start monitoring mailboxes in %d second(s)", delay);
+	else
+		g_message ("Start monitoring mailboxes now");
+#endif
+	std::vector<class Mailbox *>::iterator mailbox;
+
+	g_mutex_lock (mutex_);
+	mailbox = mailbox_.begin ();
+	while (mailbox != mailbox_.end ())
+	  (*(mailbox++))->threaded_start (delay);
+	g_mutex_unlock (mutex_);
+}
+
+/**
+ *  Stop monitoring all mailboxes.
+ */
+void 
+Biff::stop_monitoring (void)
+{
+#ifdef DEBUG
+	g_message ("Stop monitoring mailboxes");
+#endif
+	std::vector<class Mailbox *>::iterator mailbox;
+
+	g_mutex_lock (mutex_);
+	mailbox = mailbox_.begin ();
+	while (mailbox != mailbox_.end ())
+	  (*(mailbox++))->stop ();
+	g_mutex_unlock (mutex_);
+}
+
+// ============================================================================
+//  options
+// ============================================================================
+
 /**
  *  This function is called when an option is changed that has the
  *  OPTFLG_CHANGE flag set.
@@ -363,7 +545,7 @@ Biff::option_changed (Option *option)
 
 	// UI_MODE
 	if (option->name() == "ui_mode") {
-		value ("gtk_mode", ((Option_UInt *)option)->value() == GTK_MODE);
+		value ("gtk_mode", ((Option_UInt *)option)->value() == MODE_GTK);
 		return;
 	}
 }
@@ -379,10 +561,6 @@ Biff::option_update (Option *option)
 {
 }
 
-// ================================================================================
-//  i/o
-// ================================================================================
-
 /**
  *  The loaded config file belongs to an old version of gnubiff. All options
  *  with changed default values will be converted if possible. If the option
@@ -409,9 +587,8 @@ Biff::upgrade_options (void)
 
 	// Check for newer version config file
 	if (version > gnubiff_version) {
-// Note: No new strings in gnubiff 2.1.x
-//		g_warning (_("Loaded config file from newer gnubiff version \"%s\"."),
-//					 config_version.c_str ());
+		g_warning (_("Loaded config file from newer gnubiff version \"%s\"."),
+					 config_version.c_str ());
 		return; 
 	}
 	if (version == gnubiff_version)
@@ -436,7 +613,7 @@ Biff::upgrade_options (void)
 	}
 
 	// Mailbox options
-	for (guint i = 0; i < size(); i++) {
+	for (guint i = 0; i < get_number_of_mailboxes (); i++) {
 		Mailbox *mb = mailbox(i);
 
 		// Option: ADDRESS (for maildir protocol)
@@ -451,6 +628,15 @@ Biff::upgrade_options (void)
 				g_free (base);
 			}
 		}
+
+#ifdef USE_PASSWORD
+		// Option: PASSWORD
+		if (version < 2002000) {
+			std::string passphrase = value_string ("passphrase");
+			mb->password (Decoding::decrypt_password_legacy (passphrase,
+															 mb->password ()));
+		}
+#endif
 	}
 
 	// End message
@@ -464,6 +650,10 @@ Biff::upgrade_options (void)
 	}
 }
 
+// ============================================================================
+//  i/o
+// ============================================================================
+
 /**
  * Opens the new block {\em name} of options in the configuration file.
  *
@@ -550,17 +740,9 @@ Biff::save (void)
 	std::map<std::string,std::string> name_value;
 	g_mutex_lock (mutex_);
 	for (unsigned int i=0; i< mailbox_.size(); i++) {
-#ifdef USE_PASSWORD
-		// Encrypt password
-		mailbox_[i]->value ("password", Decoding::encrypt_password (mailbox_[i]->value_string ("password"), passtable_));
-#endif
 		// Save options
 		mailbox_[i]->to_strings (OPTGRP_MAILBOX, name_value);
 		save_parameters (name_value, "mailbox");
-#ifdef USE_PASSWORD
-		// Decrypt password
-		mailbox_[i]->value ("password", Decoding::decrypt_password (mailbox_[i]->value_string ("password"), passtable_));
-#endif
 	}
 	g_mutex_unlock (mutex_);
 
@@ -645,7 +827,7 @@ Biff::load (void)
 
 	file.close ();
 
-	// Do we have an old config file?
+	// Do we have an config file from another version?
 	if (value_string ("version") != PACKAGE_VERSION)
 		upgrade_options ();
 
@@ -751,10 +933,6 @@ Biff::xml_end_element (GMarkupParseContext *context,
 
 		// Get options
 		mailbox_[pos]->from_strings (OPTGRP_MAILBOX, buffer_load_);
-#ifdef USE_PASSWORD
-		// Decrypt password
-		mailbox_[pos]->value ("password", Decoding::decrypt_password (mailbox_[pos]->value_string ("password"), passtable_));
-#endif
 	}
 	// Options common to all mailboxes
 	else
