@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: decoding.cc,v $
-// Revision      : $Revision: 1.20 $
-// Revision date : $Date: 2005/05/07 17:07:48 $
+// Revision      : $Revision: 1.25 $
+// Revision date : $Date: 2006/01/15 23:12:58 $
 // Author(s)     : Nicolas Rougier, Robert Sowada
 // Short         : Various functions for decoding, converting ...
 //
@@ -73,15 +73,21 @@ Decoding::decode_body (std::vector<std::string> &mail, std::string encoding,
 	// 7bit, 8bit encoding: nothing to do
 	if ((encoding=="7bit") || (encoding=="8bit")); // || (encoding=="binary"));
 	// Quoted-Printable
-	else if (encoding=="quoted-printable") {
+	else if (encoding == "quoted-printable") {
 		std::vector<std::string> decoded=decode_quotedprintable(mail, bodypos);
-		mail.erase(mail.begin()+bodypos, mail.end());
-		for (std::string::size_type i = 0; i < decoded.size(); i++)
-			mail.push_back(decoded[i]);
+		mail.erase (mail.begin() + bodypos, mail.end());
+		for (guint i = 0; i < decoded.size(); i++)
+			mail.push_back (decoded[i]);
+	}
+	// Base64
+	else if (encoding == "base64") {
+		std::string decoded = decode_base64 (mail, bodypos);
+		mail.erase (mail.begin() + bodypos, mail.end());
+		mail.push_back (decoded);
 	}
 	// Unknown encoding: Replace body text by a error message
 	else {
-		mail.erase (mail.begin()+bodypos, mail.end());
+		mail.erase (mail.begin() + bodypos, mail.end());
 		gchar *tmp = g_strdup_printf (_("[The encoding \"%s\" of this mail "
 										"can't be decoded]"),
 									  encoding.c_str());
@@ -374,6 +380,35 @@ Decoding::decode_base64 (const std::string &todec)
 	return result;
 }
 
+/**
+ * Decoding of a vector of base64 encoded strings. It is assumed that
+ * each string in the vector {\em todec} represents one line of the
+ * base64 encoded text. Lines that have an invalid encoding are
+ * omitted.
+ *
+ * Note: The last character of the decoded string may not be validly encoded
+ * in its charset.
+ *
+ * See RFC 3548, 2045 6.8 for the definition of base64 encoding.
+ *
+ * @param todec  Vector of base64 encoded C++ strings that will be decoded.
+ * @param pos    Number of the first line in the vector that shall be decoded.
+ *               The default value is 0.
+ * @return       String consisting of the decoded text
+ */
+std::string 
+Decoding::decode_base64 (const std::vector<std::string> &todec,
+						 std::string::size_type pos)
+{
+	// Append all lines
+	std::string body;
+	while (pos < todec.size ())
+		body += todec[pos++];
+
+	// Decode line
+	return decode_base64 (body);
+}
+
 /**
  * Decoding of a q-encoded strings. Q-Encoding is similar to quoted-printable
  * encoding and is used in mail headers. See RFC 2047 4.2. for a definition.
@@ -464,10 +499,11 @@ Decoding::decode_quotedprintable (const std::string &todec)
  *
  * Note: For mail headers q-encoding is used instead of quoted-printable.
  *
- * @param todec  Reference to a C++ vector of C++ strings that will be decoded.
- * @param pos    Number of the first line in the vector that has to be decoded.
+ * @param todec  Vector of quoted printable encoded C++ strings that will be
+ *               decoded.
+ * @param pos    Number of the first line in the vector that shall be decoded.
  *               The default value is 0.
- * @return       C++ vector of C++ strings consisting of the decoded text
+ * @return       Vector of C++ strings consisting of the decoded text
  */
 std::vector<std::string> 
 Decoding::decode_quotedprintable (const std::vector<std::string> &todec,
@@ -635,16 +671,23 @@ Decoding::utf8_to_imaputf7 (const gchar *str, gssize len)
 /**
  *  Convert the string {\em text} from the character set {\em charset} to
  *  utf-8. If no character set is given the string is assumed to be in the
- *  C runtime character set. If the string cannot be converted a error message
- *  is returned.
+ *  C runtime character set.
+ *
+ *  If the string cannot be converted and if {\em retries} is non
+ *  zero, the string {\em text} without the last byte is converted (if
+ *  possible). This is done because gnubiff cannot know the end of
+ *  characters when decoding certain encodings (e.g. base64). If
+ *  {\em retries} is zero an error message is returned.
  *
  *  @param  text     String to be converted
  *  @param  charset  Character set of the string {\em text} or empty
+ *  @param  retries  Maximum number of retries. The default is 0.
  *  @return          Converted string or error message (as character array).
  *                   This string has to be freed with g_free().
  */
 gchar * 
-Decoding::charset_to_utf8 (std::string text, std::string charset)
+Decoding::charset_to_utf8 (std::string text, std::string charset,
+						   guint retries)
 {
 	gchar *utf8 = (gchar *) text.c_str();
 	if (!charset.empty())
@@ -652,6 +695,12 @@ Decoding::charset_to_utf8 (std::string text, std::string charset)
 	else
 		utf8 = g_locale_to_utf8 (text.c_str(), -1, 0, 0, 0);
 
+	// Could not convert. Retry?
+	if (!utf8 && retries)
+		return charset_to_utf8 (text.substr (0, text.size()-1), charset,
+								retries - 1);
+
+	// Could not convert at all!
 	if (!utf8) {
 		gchar *tmp = g_strdup_printf (_("[Cannot convert character sets "
 										"(from \"%s\" to \"utf-8\")]"),
@@ -680,59 +729,30 @@ Decoding::ascii_strdown (const std::string &str)
 }
 
 /**
- * Encrypt the password for saving.
- * Pop3 and Imap4 protocols require password in clear so we have to
- * save passwords in clear within the configuration file. No need to say
- * this is higly unsecure if somebody looks at the file. So we try to
- * take some measures:
- * \begin{itemize}
- *    \item The file is made readable by owner only.
- *	  \item Password is "crypted" so it's not directly human readable.
- * \end{\itemize}
- * Of course, these measures won't prevent a determined person to break
- * in but will at least prevent "ordinary" people to be able to steal
- * password easily.
+ *  Decrypt a password that was stored before version 2.2.0.
  *
- * If no password saving is selected at configure time, an empty string is
- * returned.
+ *  If no password saving is selected at configure time, an empty string is
+ *  returned.
  *
- * @param password  Password to be encrypted
- * @param passtable Passtable to be used for the encryption
- * @return          "Encrypted" password or empty string
+ *  @param  passphrase Passphrase to be used for decryption
+ *  @param  password   Encrypted password
+ *  @return            Decrypted password
  */
 std::string 
-Decoding::encrypt_password (const std::string &password,
-							const std::string &passtable)
+Decoding::decrypt_password_legacy (const std::string &passphrase,
+								   const std::string &password)
 {
 #ifdef USE_PASSWORD
-	std::stringstream encrypted;
+	// Build passtable
+	std::string passtable = passphrase;
+	std::string buffer;
+	for (std::string::size_type i = 0; i < passtable.size(); i++)
+		if (buffer.find(passtable[i]) == std::string::npos)
+			buffer += passtable[i];
+	passtable = buffer;
 
-	for (std::string::size_type j = 0; j < password.size(); j++)
-		encrypted << passtable[password[j]/16] << passtable[password[j]%16];
-	return encrypted.str();
-#else
-	return std::string("");
-#endif
-}
-
-/**
- * Decrypt a password from the config file.
- *
- * If no password saving is selected at configure time, an empty string is
- * returned.
- *
- * @param password  Password to be decrypted
- * @param passtable Passtable to be used for the decryption
- * @return          Decrypted password or empty string
- * @see             Decoding::crypt_password()
- */
-std::string 
-Decoding::decrypt_password (const std::string &password,
-							const std::string &passtable)
-{
-#ifdef USE_PASSWORD
+	// Decrypt password
 	std::stringstream decrypted;
-
 	for (std::string::size_type i = 0; i+1 < password.size(); i += 2) {
 		char c = 0;
 		for (guint j = 0; j < 16; j++) {
@@ -748,3 +768,107 @@ Decoding::decrypt_password (const std::string &password,
 	return std::string("");
 #endif
 }
+
+/**
+ *  Decrypt the AES encrypted data {\em data} using the passphrase
+ *  {\em passphrase}. The 128 bit AES algorithm is used.
+ *
+ *  Note: If no AES libraries are available or if an error occurs, this
+ *        function returns an empty string.
+ *
+ *  @param  passphrase Passphrase to be used for decryption (must be 16
+ *                     characters long)
+ *  @param  data       Encrypted data
+ *  @return            Decrypted data
+ */
+std::string 
+Decoding::decrypt_aes (const std::string &passphrase, const std::string &data)
+{
+#ifdef HAVE_AES
+	// Check Passphrase
+	if (passphrase.size() < 16)
+		return std::string ("");
+
+	// Determine size and allocate memory
+	guint size = data.size()/2;
+	if (size == 0)
+		return std::string ("");
+	unsigned char *bin = new unsigned char[size+1];
+	if (!bin)
+		return std::string ("");
+	unsigned char *result = new unsigned char[size+1];
+	if (!result)
+		return std::string ("");
+
+	// ASCII to Binary
+	for (guint i = 0; i < size; i++)
+		bin[i] = 16*g_ascii_xdigit_value (data[2*i])
+				 +  g_ascii_xdigit_value (data[2*i+1]);
+	bin[size] = '\0';
+
+	// Decrypt via AES
+	AES_KEY aes_key;
+	AES_set_decrypt_key ((unsigned char *)passphrase.c_str (), 128, &aes_key);
+	AES_decrypt (bin, result, &aes_key);
+
+	// Free memory
+	std::string result_str = std::string ((char *)result);
+	delete (bin);
+	delete (result);
+
+	return result_str;
+#else
+	return std::string ("");
+#endif
+}
+
+/**
+ *  Encrypt the AES encrypted data {\em data} using the passphrase
+ *  {\em passphrase}. The 128 bit AES algorithm is used.
+ *
+ *  Note: If no AES libraries are available or if an error occurs, this
+ *        function returns an empty string.
+ *
+ *  @param  passphrase Passphrase to be used for encryption (must be 16
+ *                     characters long)
+ *  @param  data       Decrypted data
+ *  @return            Encrypted data
+ */
+std::string 
+Decoding::encrypt_aes (const std::string &passphrase, const std::string &data)
+{
+#ifdef HAVE_AES
+	const char hex[] = "0123456789ABCDEF";
+
+	// Check Passphrase
+	if (passphrase.size() < 16)
+		return std::string ("");
+
+	// Determine size and allocate memory
+	guint size = (data.size()+15)/16*16;
+	if (size == 0)
+		return std::string ("");
+	unsigned char *result = new unsigned char[2*size];
+	if (!result)
+		return std::string ("");
+
+	// Encrypt via AES
+	AES_KEY aes_key;
+	AES_set_encrypt_key ((unsigned char *)passphrase.c_str (), 128, &aes_key);
+	AES_encrypt ((unsigned char *)data.c_str (), result, &aes_key);
+
+	// Binary to ASCII
+	for (guint i = size; i > 0; i--) {
+		result[2*i-1] = hex[result[i-1]&0x0f];
+		result[2*i-2] = hex[result[i-1]/16];
+	}
+
+	// Free memory
+	std::string result_str = std::string ((char *)result, 2*size);
+	delete (result);
+
+	return result_str;
+#else
+	return std::string ("");
+#endif
+}
