@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: gnubiff_options.cc,v $
-// Revision      : $Revision: 1.32 $
-// Revision date : $Date: 2005/08/07 14:28:23 $
+// Revision      : $Revision: 1.48 $
+// Revision date : $Date: 2006/05/07 22:07:53 $
 // Author(s)     : Robert Sowada, Nicolas Rougier
 // Short         : Options for gnubiff
 //
@@ -45,33 +45,39 @@ const gchar *Gnubiff_Options::protocol_gchar[] = {
 /**
  *  Add options of the given groups.
  *
- *  @param  groups  Groups of which the options shall be added.
+ *  @param  groups     Groups of which the options shall be added.
+ *  @param  deprecated Shall deprecated options be added? The default is
+ *                     false.
  */
 void 
-Gnubiff_Options::add_options (guint groups)
+Gnubiff_Options::add_options (guint groups, gboolean deprecated)
 {
 	if (groups & OPTGRP_APPLET)
-		add_options_applet ();
+		add_options_applet (deprecated);
 
 	if (groups & OPTGRP_GENERAL)
-		add_options_general ();
+		add_options_general (deprecated);
 
 	if (groups & OPTGRP_INFORMATION)
-		add_options_information ();
+		add_options_information (deprecated);
 
 	if (groups & OPTGRP_MAILBOX)
-		add_options_mailbox ();
+		add_options_mailbox (deprecated);
 
 	if (groups & OPTGRP_POPUP)
-		add_options_popup ();
+		add_options_popup (deprecated);
 
 	if (groups & OPTGRP_SECURITY)
-		add_options_security ();
+		add_options_security (deprecated);
 }
 
-/// Add options for the appearance of the applet
+/**
+ *  Add options for the appearance of the applet.
+ *
+ *  @param  deprecated  Shall deprecated options be added?
+ */
 void 
-Gnubiff_Options::add_options_applet (void)
+Gnubiff_Options::add_options_applet (gboolean deprecated)
 {
 	add_group (new Option_Group ("applet", OPTGRP_APPLET,
 		"Appearance of the applet."));
@@ -161,11 +167,17 @@ Gnubiff_Options::add_options_applet (void)
 		"Font to be used in the applet.",
 								   "sans 10", OPTFLG_NONE, OPTGUI_FONT,
 								   "applet_font_button"));
+	if (!deprecated)
+		return;
 }
 
-/// Add general options
+/**
+ *  Add general options
+ *
+ *  @param  deprecated  Shall deprecated options be added?
+ */
 void 
-Gnubiff_Options::add_options_general (void)
+Gnubiff_Options::add_options_general (gboolean deprecated)
 {
 	add_group (new Option_Group ("general", OPTGRP_GENERAL,
 		"General options."));
@@ -269,15 +281,54 @@ Gnubiff_Options::add_options_general (void)
 		"Directory in which to look for certificates when building the "
 		"certificate chain.",
 								   "/etc/ssl/certs/"));
+	// SIGNAL_SIGUSR1
+	const static guint i7[] = {SIGNAL_NONE, SIGNAL_MARK_AS_READ, SIGNAL_START,
+							   SIGNAL_STOP, SIGNAL_POPUP_ENABLE,
+							   SIGNAL_POPUP_DISABLE, SIGNAL_POPUP_TOGGLE, 0};
+	const static gchar *s7[] = {"none", "mark_as_read", "start", "stop",
+								"popup_enable", "popup_disable",
+								"popup_toggle", NULL};
+	add_option (new Option_UInt ("signal_sigusr1", OPTGRP_GENERAL,
+		"Action to be executed if the signal SIGUSR1 is caught by gnubiff. "
+		"\nPossible actions are ignoring the signal (\"none\"), marking all "
+		"messages as read (\"mark_as_read\"), starting monitoring "
+		"(\"start\"), stopping monitoring (\"stop\"), enabling and disabling "
+        "the popup (\"popup_enable\" and \"popup_disable\") or "
+		"toggling the popup status (\"popup_toggle\").",
+								 SIGNAL_MARK_AS_READ,
+								 OPTFLG_ID_INT_STRICT, i7,s7));
+	// SIGNAL_SIGUSR2
+	add_option (new Option_UInt ("signal_sigusr2", OPTGRP_GENERAL,
+		"Action to be executed if the signal SIGUSR2 is caught by gnubiff. "
+		"\nFor the available actions see the description of option "
+		"SIGNAL_SIGUSR1.",
+								 SIGNAL_NONE,
+								 OPTFLG_ID_INT_STRICT, i7,s7));
+	// STARTUP_PREFERENCES
+	add_option (new Option_Bool ("startup_preferences", OPTGRP_GENERAL,
+		"Shall the preferences dialog be shown when starting gnubiff? "
+		"This option is ignored in gnome mode.",
+								 true));
+	if (!deprecated)
+		return;
 }
 
-/// Add options that are for information purposes only
+/**
+ *  Add options that are for information purposes only.
+ *
+ *  @param  deprecated  Shall deprecated options be added?
+ */
 void 
-Gnubiff_Options::add_options_information (void)
+Gnubiff_Options::add_options_information (gboolean deprecated)
 {
 	add_group (new Option_Group ("information", OPTGRP_INFORMATION,
 		"Not to be changed, for information purposes only."));
 
+	// CONFIG_FILE_LOADED
+	add_option (new Option_Bool ("config_file_loaded", OPTGRP_INFORMATION,
+		"Has the configuration file been loaded?",
+								 false,
+								 OPTFLG_NOSAVE | OPTFLG_AUTO | OPTFLG_NOSHOW));
 	// GTK_MODE
 	const static gchar *s1[] = {"applet_geometry_check",
 								"applet_geometry_entry",
@@ -293,11 +344,12 @@ Gnubiff_Options::add_options_information (void)
 								 |OPTFLG_FIXED | OPTFLG_NOSAVE | OPTFLG_NOSHOW,
 								 protocol_int, protocol_gchar));
 	// UI_MODE
-	const static guint i2[] = {GTK_MODE, GNOME_MODE, 0};
-	const static gchar *s2[] = {"gtk", "gnome", NULL};
+	const static guint i2[] = {MODE_GTK, MODE_GNOME, MODE_NOGUI,
+							   MODE_SYSTEMTRAY, 0};
+	const static gchar *s2[] = {"gtk", "gnome", "nogui", "systemtray", NULL};
 	add_option (new Option_UInt ("ui_mode", OPTGRP_INFORMATION,
 		"User interface mode in which gnubiff is running.",
-								 GTK_MODE, OPTFLG_CHANGE | OPTFLG_ID_INT_STRICT
+								 MODE_GTK, OPTFLG_CHANGE | OPTFLG_ID_INT_STRICT
 								 | OPTFLG_FIXED | OPTFLG_NOSAVE
 								 | OPTFLG_NOSHOW, i2, s2));
 	// VERSION
@@ -308,11 +360,17 @@ Gnubiff_Options::add_options_information (void)
 		"that were changed by the user usually have to be converted manually.",
 								   PACKAGE_VERSION,
 								   OPTFLG_FIXED | OPTFLG_NOSHOW));
+	if (!deprecated)
+		return;
 }
 
-/// Add options that are different for each mailbox.
+/**
+ *  Add options that are different for each mailbox.
+ *
+ *  @param  deprecated  Shall deprecated options be added?
+ */
 void 
-Gnubiff_Options::add_options_mailbox (void)
+Gnubiff_Options::add_options_mailbox (gboolean deprecated)
 {
 	add_group (new Option_Group ("mailbox", OPTGRP_MAILBOX,
 		"Options that are mailbox dependant."));
@@ -355,7 +413,7 @@ Gnubiff_Options::add_options_mailbox (void)
 								   "name_entry"));
 	// DELAY
 	add_option (new Option_UInt ("delay", OPTGRP_MAILBOX,
-		"Time interval between mail checks for network mailboxes when "
+		"Time interval between mail checks for mailboxes that use "
 		"polling (in seconds).",
 								 180, OPTFLG_CHANGE));
 	// DELAY_MINUTES
@@ -393,12 +451,18 @@ Gnubiff_Options::add_options_mailbox (void)
 		"Password of the mailbox. This is needed to login into network "
 		"mailboxes.",
 								   "",
-#ifdef USE_PASSWORD
-								   OPTFLG_NONE,
-#else
-								   OPTFLG_NOSAVE,
-#endif
+								   OPTFLG_NOSAVE | OPTFLG_NOSHOW,
 								   OPTGUI_ENTRY, "password_entry"));
+	// PASSWORD_AES
+	add_option (new Option_String ("password_aes", OPTGRP_MAILBOX,
+		"AES encrypted version of this mailbox's password.",
+								   "",
+#ifdef USE_PASSWORD
+								   OPTFLG_CHANGE | OPTFLG_UPDATE
+#else
+								   OPTFLG_NOSAVE | OPTFLG_NOSHOW
+#endif
+								   ));
 	// USE_OTHER_PORT
 	const static gchar *s2[] = {"port_spin", NULL};
 	add_option (new Option_Bool ("use_other_port", OPTGRP_MAILBOX,
@@ -471,11 +535,33 @@ Gnubiff_Options::add_options_mailbox (void)
 		"being noticed because of race conditions.\n"
 		"This option is used for the file protocol only.",
 								 false));
+	// LOCAL_FAM_ENABLE
+	add_option (new Option_Bool ("local_fam_enable", OPTGRP_MAILBOX,
+		"Shall the mailbox be monitored by the FAM demon (or a substitution "
+		"like GAMIN)? If this option is set to false, polling will be used "
+		"instead. The time between two checks is determined by the "
+		"\"delay\" option of this mailbox.\n"
+		"Note: If no support for FAM is compiled into the gnubiff binary "
+        "this option is ignored and polling is always used.\n"
+		"This option is used for the local protocols only.",
+								 true));
+	// ERROR_RESET_MSGS
+	add_option (new Option_Bool ("error_reset_msgs", OPTGRP_MAILBOX,
+		"Shall the read status of all messages be reset to unread if there is "
+		"a failure in retrieving the mailbox's messages? This results in all "
+		"messages be presented as new once messages can be obtained again.",
+								 false));
+	if (!deprecated)
+		return;
 }
 
-/// Add options for the appearance of the popup
+/**
+ *  Add options for the appearance of the popup
+ *
+ *  @param  deprecated  Shall deprecated options be added?
+ */
 void 
-Gnubiff_Options::add_options_popup (void)
+Gnubiff_Options::add_options_popup (gboolean deprecated)
 {
 	add_group (new Option_Group ("popup", OPTGRP_POPUP,
 		"Appearance of the popup."));
@@ -573,6 +659,15 @@ Gnubiff_Options::add_options_popup (void)
 		"Maximum number of mail body lines that will be displayed in the "
 		"popup.",
 								 10));
+	// POPUP_CONVERT_RETRIES
+	add_option (new Option_UInt ("popup_convert_retries", OPTGRP_POPUP,
+		"Maximum number of retries when converting a message's body to UTF-8. "
+		"Because gnubiff does not fetch the whole body but only the first "
+		"lines the last character may be invalid after decoding from certain "
+		"encodings (e.g. base64). This option gives the maximum number of "
+		"retries when trying to convert such strings to get a validly "
+		"encoded string.",
+								 5));
 	// POPUP_SORT_BY
 	add_option (new Option_String ("popup_sort_by", OPTGRP_POPUP,
 		"A space separated list of header properties by which the headers "
@@ -589,11 +684,17 @@ Gnubiff_Options::add_options_popup (void)
 		"The sorting order of each property can be reversed by prefixing a "
 		"\"!\".",
 								   "!position mailbox"));
+	if (!deprecated)
+		return;
 }
 
-/// Add options that affect security issues.
+/**
+ *  Add options that affect security issues.
+ *
+ *  @param  deprecated  Shall deprecated options be added?
+ */
 void 
-Gnubiff_Options::add_options_security (void)
+Gnubiff_Options::add_options_security (gboolean deprecated)
 {
 	add_group (new Option_Group ("security", OPTGRP_SECURITY,
 		"Options that affect security issues. Most of these options help "
@@ -612,6 +713,18 @@ Gnubiff_Options::add_options_security (void)
 		"mailbox.",
 								 100, OPTFLG_NONE, OPTGUI_SPIN,
 								 "max_mail_spin"));
+#ifdef USE_PASSWORD
+	// PASSPHRASE
+	add_option (new Option_String ("passphrase", OPTGRP_SECURITY,
+		"This is the password phrase used for the encryption of the passwords "
+		"in the configuration file (if this feature has been enabled at "
+		"configuration time).\n"
+		"Note: This phrase is stored in the gnubiff binary and can so "
+		"obtained by anyone that can read it.",
+								   PASSWORD_STRING"FEDCBA9876543210",
+								   OPTFLG_FIXED | OPTFLG_AUTO
+								   | OPTFLG_NOSAVE ));
+#endif
 	// PREVDOS_ADDITIONAL_LINES
 	add_option (new Option_UInt ("prevdos_additional_lines", OPTGRP_SECURITY,
 		"Maximum number of lines that are read from the network additionally "
@@ -672,4 +785,6 @@ Gnubiff_Options::add_options_security (void)
         "standard.\n"
 		"This option is used for the POP3 protocol.",
 								 70));
+	if (!deprecated)
+		return;
 }
