@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,9 +19,9 @@
 // ========================================================================
 //
 // File          : $RCSfile: gui.cc,v $
-// Revision      : $Revision: 1.6 $
-// Revision date : $Date: 2005/04/04 22:08:28 $
-// Author(s)     : Nicolas Rougier
+// Revision      : $Revision: 1.10 $
+// Revision date : $Date: 2006/02/12 17:52:09 $
+// Author(s)     : Nicolas Rougier, Robert Sowada
 // Short         : 
 //
 // This file is part of gnubiff.
@@ -132,11 +132,11 @@ extern "C" {
 		int accept_id = GPOINTER_TO_INT (user_data);
 		if (response_id == accept_id) {
 			char *uri;
-			uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog));
+			uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER(dialog));
 			if (uri)
 				g_free(uri);
 			else
-				g_signal_stop_emission_by_name(dialog, "response");
+				g_signal_stop_emission_by_name (dialog, "response");
 		}
 	}
 
@@ -176,7 +176,7 @@ GUI::~GUI (void)
 }
 
 gint
-GUI::create (void)
+GUI::create (gpointer callbackdata)
 {
 	if (xml_)
 		return true;
@@ -206,7 +206,7 @@ GUI::create (void)
 		exit (1);
 	}
 
-	glade_xml_signal_autoconnect_full (xml_, GUI_connect, this);
+	glade_xml_signal_autoconnect_full (xml_, GUI_connect, callbackdata);
 	create_insert_version ();
 	return true;
 }
@@ -215,7 +215,10 @@ GUI::create (void)
  *  Insert version and date information into certain widgets. For widgets
  *  called "gnubiff" (if existing) the following substitutions are made:
  *  "%v" is substituted by the version number, "%c" is substituted by "CVS" if
- *  it's a cvs version of gnubiff and "" otherwise..
+ *  it's a cvs version of gnubiff and "" otherwise.
+ *
+ *  The version information for the about dialog is also set if this dialog
+ *  is present.
  */
 void 
 GUI::create_insert_version (void)
@@ -237,7 +240,15 @@ GUI::create_insert_version (void)
 		if (!text)
 			continue;
 		std::string newtext = substitute (text, chars, toinsert);
-		gtk_label_set_label (label, newtext.c_str());
+		gtk_label_set_label (label, newtext.c_str ());
+	}
+
+	// About dialog
+	GtkAboutDialog *about;
+	about = GTK_ABOUT_DIALOG (glade_xml_get_widget (xml_, "gnubiffabout"));
+	if (about) {
+		std::string newtext = substitute ("%v %c", chars, toinsert);
+		gtk_about_dialog_set_version (about, newtext.c_str ());
 	}
 }
 
