@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,9 +19,9 @@
 // ========================================================================
 //
 // File          : $RCSfile: mailbox.cc,v $
-// Revision      : $Revision: 1.79 $
-// Revision date : $Date: 2005/06/23 20:11:48 $
-// Author(s)     : Nicolas Rougier
+// Revision      : $Revision: 1.87 $
+// Revision date : $Date: 2006/02/03 22:41:20 $
+// Author(s)     : Nicolas Rougier, Robert Sowada
 // Short         : 
 //
 // This file is part of gnubiff.
@@ -59,25 +59,25 @@ Mailbox::Mailbox (Biff *biff)
 	stopped_ = false;
 
 	// Add options
-	add_options (OPTGRP_MAILBOX);
+	add_options (OPTGRP_MAILBOX, !biff_->value_bool ("config_file_loaded"));
 
 	// Set session specific options
 	value ("uin", uin_count_++);
 
 	timetag_ = 0;
-	mutex_ = g_mutex_new();
-	monitor_mutex_ = g_mutex_new();
+	mutex_ = g_mutex_new ();
+	monitor_mutex_ = g_mutex_new ();
 }
 
 Mailbox::Mailbox (const Mailbox &other)
 {
-	biff_			  = other.biff_;
+	biff_ = other.biff_;
 	add_option ((Mailbox &)other);
 
 	status (MAILBOX_UNKNOWN);
 	timetag_= 0;
-	mutex_ = g_mutex_new();
-	monitor_mutex_ = g_mutex_new();
+	mutex_ = g_mutex_new ();
+	monitor_mutex_ = g_mutex_new ();
 }
 
 Mailbox &
@@ -170,15 +170,24 @@ Mailbox::fetch (void)
 	// nothing to do, this mailbox is virtual
 }
 
-void
-Mailbox::read (gboolean value)
+/**
+ *  Mark all collected messages in this mailbox as read. The status of
+ *  uncollected messages isn't changed.
+ */
+void 
+Mailbox::mark_messages_as_read (void)
 {
-	if ((!value) || (!g_mutex_trylock (mutex_)))
+	if (!g_mutex_trylock (mutex_))
 		return;
+
+	// All seen messages should be hidden; no unread messages
 	hidden_.clear();
 	hidden_ = seen_;
 	unread_.clear();
-	biff_->save();
+
+	// FIXME: If there might be unread messages that haven't been fetched yet,
+	// they should be read now. Bug reported by Maik Wachsmuth
+
 	g_mutex_unlock (mutex_);
 }
 
@@ -276,6 +285,16 @@ Mailbox::option_changed (Option *option)
 		return;
 	}
 
+#ifdef USE_PASSWORD
+	// PASSWORD_AES
+	if (option->name() == "password_aes") {
+		std::string decpass = decrypt_aes (biff_->value_string ("passphrase"),
+										   ((Option_String *)option)->value());  
+		value ("password", decpass);
+		return;
+	}
+#endif
+
 	// SEEN
 	if (option->name() == "seen") {
 		get_values ("seen", hidden_, true, false);
@@ -306,6 +325,16 @@ Mailbox::option_update (Option *option)
 	if (!option)
 		return;
 
+#ifdef USE_PASSWORD
+	// PASSWORD_AES
+	if (option->name() == "password_aes") {
+		std::string enc_aes = encrypt_aes (biff_->value_string ("passphrase"),
+										   value_string ("password"));
+		((Option_String *)option)->value (enc_aes);
+		return;
+	}
+#endif
+
 	// SEEN
 	if (option->name() == "seen") {
 		set_values ("seen", hidden_, true, false);
@@ -476,7 +505,7 @@ Mailbox::lookup (void)
 	// After replace, "this" is destroyed so we must return immediately
 	if (mailbox) {
 		g_mutex_unlock (monitor_mutex_);
-		biff_->replace (this, mailbox);
+		biff_->replace_mailbox (this, mailbox);
 		return;
 	}
 
@@ -985,6 +1014,23 @@ Mailbox::start_checking (void)
 	update_mailbox_status ();
 }
 
+/**
+ *  Set the mailbox status into error state. Also resets the read status of
+ *  all messages if the corresponding mailbox option is set to true.
+ */
+void 
+Mailbox::set_status_mailbox_error (void)
+{
+	// Set mailbox status
+	status (MAILBOX_ERROR);
+
+	// Reset read status of messages if wanted by the user
+	if (value_bool ("error_reset_msgs")) {
+		unread_.clear ();
+		seen_.clear ();
+	}
+}
+
 /**
  * Decide whether a mail has to be fetched and parsed. If the mail is already
  * known it is inserted into the new_unread_ map. So it can be displayed later.
@@ -1041,3 +1087,47 @@ Mailbox::find_mail (std::string mailid, Header &mail)
 
 	return ok;
 }
+
+// ============================================================================
+//  main -- messages
+// ============================================================================
+
+/**
+ *  Add headers of messages in the mailbox to the vector {\em headers}. If
+ *  {\em use_max_num} is true at most {\em max_num} headers are added. If
+ *  {\em empty} is true, the vector will be emptied before adding vectors.
+ *
+ *  @param  headers      Vector of headers to which the headers will be added.
+ *  @aram  use_max_num  If true only a limited amount of headers will be
+ *                       added (the default is false).
+ *  @param  max_num      If {\em use_max_num} is true, this is the maximum
+ *                       number of headers to be added (the default is 0).
+ *  @param  empty        If true the vector will be emptied (the default is
+ *                       {\em false}.
+ */
+void 
+Mailbox::get_message_headers (std::vector<Header *> &headers,
+							  gboolean use_max_num, guint max_num,
+							  gboolean empty)
+{
+	guint m = 0;
+
+	// Empty headers
+	if (empty)
+		headers.clear ();
+
+	g_mutex_lock (mutex_);
+
+	std::map<std::string, Header>::iterator ie, i;
+	i  = unread_.begin ();
+	ie = unread_.end ();
+	if (use_max_num)
+		m = unread_.size() - max_num;
+	while (i != ie) {
+		if (i->second.position() > m)
+			headers.push_back (new Header(i->second));
+		i++;
+	}
+
+	g_mutex_unlock (mutex_);
+}
