@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,9 +19,9 @@
 // ========================================================================
 //
 // File          : $RCSfile: mailbox.h,v $
-// Revision      : $Revision: 1.53 $
-// Revision date : $Date: 2005/06/23 20:12:05 $
-// Author(s)     : Nicolas Rougier
+// Revision      : $Revision: 1.57 $
+// Revision date : $Date: 2006/02/03 22:41:20 $
+// Author(s)     : Nicolas Rougier, Robert Sowada
 // Short         : 
 //
 // This file is part of gnubiff.
@@ -142,12 +142,13 @@ public:
 	virtual void start (void);						// start method (to be overidden)
 	virtual void stop (void);						// stop method (to be overidden)
 	virtual void fetch (void);						// fetch headers (if any)
-	void read (gboolean value=true);				// mark/unmark mailbox as read
+	void mark_messages_as_read (void);
 	void lookup (void);								// try to guess mailbox format
 	static Mailbox *lookup_local(Mailbox &);        // try to guess mailbox format for a local mailbox
 	gboolean new_mail (std::string &);
 	void update_mailbox_status (void);
 	void start_checking (void);
+	void set_status_mailbox_error (void);
 	void mail_displayed (void);
 	void parse (std::vector<std::string> &mail,
 				std::string uid = std::string(""),
@@ -156,6 +157,14 @@ public:
 	guint static standard_port (guint protocol, guint auth,
 								gboolean strict = true);
 
+	// ========================================================================
+	//  main -- messages
+	// ========================================================================
+
+	void get_message_headers (std::vector<Header *> &headers,
+							  gboolean use_max_num = false, guint max_num = 0,
+							  gboolean empty = false);
+
 	// ========================================================================
 	//  access
 	// ========================================================================
