@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: option.h,v $
-// Revision      : $Revision: 1.9 $
-// Revision date : $Date: 2005/07/17 14:28:28 $
+// Revision      : $Revision: 1.11 $
+// Revision date : $Date: 2006/01/03 22:53:58 $
 // Author(s)     : Robert Sowada, Nicolas Rougier
 // Short         : One option for gnubiff
 //
@@ -73,6 +73,9 @@ const guint OPTFLG_STRINGLIST = 128;
  *  used for internal options that cannot be changed. Options that may be of
  *  information interest for the user should not be flagged.*/
 const guint OPTFLG_NOSHOW = 256;
+/** This option is deprecated. It is only present to load and convert old
+ *  configuration files.*/
+const guint OPTFLG_DEPRECATED = 512;
 
 /// Option cannot be changed by the user
 const guint OPTFLG_USER_NO_CHANGE = OPTFLG_FIXED | OPTFLG_AUTO;
