@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2005 Nicolas Rougier
+// Copyright (c) 2000-2006 Nicolas Rougier, 2004-2006 Robert Sowada
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: options.cc,v $
-// Revision      : $Revision: 1.13 $
-// Revision date : $Date: 2005/07/17 14:28:28 $
+// Revision      : $Revision: 1.17 $
+// Revision date : $Date: 2006/03/10 12:25:07 $
 // Author(s)     : Robert Sowada, Nicolas Rougier
 // Short         : Container for storing options
 //
@@ -125,6 +125,30 @@ Options::add_option (Options &options)
 	return ok;
 }
 
+/**
+ *  Remove all options that are flagged by {\em flags} from the set of
+ *  options.
+ *  
+ *  @param  flags  Flags for determining the options to be removed. It is
+ *                 sufficient for an option to be removed if it has one of the
+ *                 flags in {\em flags}.
+ *  @return        Always true.
+ */
+gboolean 
+Options::remove_options (guint flags)
+{
+	iterator opt = options_.begin ();
+
+	while (opt != options_.end ()) {
+		Option *option = opt->second;
+		if (option && ((option->flags () & flags) == flags))
+			options_.erase (opt++);
+		else
+			opt++;
+	}
+	return true;
+}
+
 /**
  *  Reset the option to the default value. This function handles the
  *  OPTFLG_CHANGE flag, so use this function instead of calling Option::reset()
@@ -576,10 +600,12 @@ Options::update_gui (OptionsGUI whattodo, Option *option, GladeXML *xml,
 	std::vector<GtkWidget *> widgets;
 	while (ss >> gui_name)
 		widgets.push_back (get_widget (gui_name.c_str(), xml, file));
+	if (!widgets.size())
+		return;
 
 	if (whattodo & OPTSGUI_GET) {
 		option->get_gui (widgets);
-		if ((option->flags() & OPTFLG_CHANGE))
+		if (option->flags() & OPTFLG_CHANGE)
 			option_changed (option);
 	}
 
