@@ -19,9 +19,9 @@
 // ========================================================================
 //
 // File          : $RCSfile: ui-authentication.cc,v $
-// Revision      : $Revision: 1.2 $
-// Revision date : $Date: 2004/12/03 17:13:39 $
-// Author(s)     : Nicolas Rougier
+// Revision      : $Revision: 1.4 $
+// Revision date : $Date: 2006/01/01 16:44:53 $
+// Author(s)     : Nicolas Rougier, Robert Sowada
 // Short         : 
 //
 // This file is part of gnubiff.
@@ -68,7 +68,7 @@ void
 Authentication::show (std::string name)
 {
 	if (!xml_)
-		create();
+		create(this);
 
 	// Try to identify mailbox by:
 	// 1. using name
